@@ -4,7 +4,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -14,7 +14,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* Runge-Kutta 4, Gaussian implicit */
@@ -191,7 +191,7 @@ rk4imp_step (double *y, rk4imp_state_t *state,
 	
 	if (s != GSL_SUCCESS)
 	  {
-	    return GSL_EBADFUNC;
+	    return s;
 	  }    
       }
       {
@@ -200,7 +200,7 @@ rk4imp_step (double *y, rk4imp_state_t *state,
 	
 	if (s != GSL_SUCCESS)
 	  {
-	    return GSL_EBADFUNC;
+	    return s;
 	  }    
       }
     }
@@ -253,7 +253,7 @@ rk4imp_apply (void *vstate,
 
       if (s != GSL_SUCCESS)
         {
-          return GSL_EBADFUNC;
+          return s;
         }
     }
 
@@ -268,7 +268,7 @@ rk4imp_apply (void *vstate,
 
     if (s != GSL_SUCCESS) 
       {
-	return GSL_EBADFUNC;
+	return s;
       }
   }
   
@@ -281,7 +281,7 @@ rk4imp_apply (void *vstate,
       {
 	/* Restore original y vector */
 	DBL_MEMCPY (y, y0_orig, dim);
-	return GSL_EBADFUNC;
+	return s;
       }
   }
 
@@ -294,7 +294,7 @@ rk4imp_apply (void *vstate,
       {
 	/* Restore original y vector */
 	DBL_MEMCPY (y, y0_orig, dim);
-	return GSL_EBADFUNC;
+	return s;
       }
   }
 
@@ -307,7 +307,7 @@ rk4imp_apply (void *vstate,
       {
 	/* Restore original y vector */
 	DBL_MEMCPY (y, y0_orig, dim);
-	return GSL_EBADFUNC;
+	return s;
       }
   }
   
@@ -320,7 +320,7 @@ rk4imp_apply (void *vstate,
       if (s != GSL_SUCCESS) {
 	/* Restore original y vector */
 	DBL_MEMCPY (y, y0_orig, dim);
-	return GSL_EBADFUNC;
+	return s;
       } 
     }
     
