@@ -140,6 +140,25 @@ gsl_rstat_sd(gsl_rstat_workspace *w)
   return (sqrt(var));
 } /* gsl_rstat_sd() */
 
+double
+gsl_rstat_rms(gsl_rstat_workspace *w)
+{
+  double mean = gsl_rstat_mean(w);
+  double sigma = gsl_rstat_sd(w);
+  double rms;
+
+  if (w->n > 0)
+    {
+      double n = (double) w->n;
+      double a = sqrt((n - 1.0) / n);
+      rms = gsl_hypot(mean, a * sigma);
+    }
+  else
+    rms = 0.0;
+
+  return rms;
+}
+
 /* standard deviation of the mean: sigma / sqrt(n) */
 double
 gsl_rstat_sd_mean(gsl_rstat_workspace *w)
@@ -188,6 +207,8 @@ gsl_rstat_kurtosis(gsl_rstat_workspace *w)
 int
 gsl_rstat_reset(gsl_rstat_workspace *w)
 {
+  int status;
+
   w->min = 0.0;
   w->max = 0.0;
   w->mean = 0.0;
@@ -196,5 +217,7 @@ gsl_rstat_reset(gsl_rstat_workspace *w)
   w->M4 = 0.0;
   w->n = 0;
 
-  return GSL_SUCCESS;
+  status = gsl_rstat_quantile_reset(w->median_workspace_p);
+
+  return status;
 } /* gsl_rstat_reset() */
